
module.exports = (opts) => {
	
	var gulp = require('gulp'),
		$ = require('gulp-load-plugins')(),
		_ = require('underscore'),
		through = require('through2'),
		merge = require('merge-stream'),
		buffer = require('vinyl-buffer');		
		Vinyl = require('vinyl'),
		htmlmin = require('htmlmin'),
		gmsmith = require('gmsmith'),
		fs = require('fs'),
		path = require('path'),
		globule = require('globule'),
		spriteTemplate = fs.readFileSync('./sprites.scss.handlebars'),
		es2015 = require('babel-preset-es2015');
		
	var options = _.extend({
		src: '../src/',
		include: [],
		fonts: {},
		responsive: [],
		
	},opts)
	
	return {
		
		// Template Engine
		
		template: (src,dest) => gulp.src(src)
			.pipe(through.obj(function(file, enc, cb) {
				var text = file.contents.toString(),
					base = path.join(file.path, '..'),
					dataFile = file.path.replace(/\.[^\.]+$/,'.json'),
					data = JSON.parse(fs.readFileSync(dataFile)),
					template = _.template(text);

				for(var i in data) {
					var t = template(data[i]),
						f = new Vinyl({
							base: base,
							path: path.join(base, i),
							contents: new Buffer(t),
						});
					console.log('template '+JSON.stringify(data[i]));
					this.push(f);
				}
				cb(null);
			}))
			.pipe(gulp.dest(dest)),
		
		
		
		
		// Images
		
		img: (src,dest) => gulp.src(src)
			.pipe($.plumber())							// error handling
			.pipe($.imagemin({progressive: true}))		// [main] : minify images
			.pipe(gulp.dest(dest)),
	
		responsive: (src,dest) => merge.apply(null, options.responsive.map((size) =>
			gulp.src(src)
				.pipe($.plumber())						// error handling
				//.pipe($.changed(dest))					// only update new files
				.pipe($.imageResize({ 					// [main] resize images
					width : size,
					height : size,
					upscale : false,
				}))
				.pipe($.imagemin({progressive: true}))	// [main] : minify images
				.pipe(gulp.dest(dest+size+'/'))
			)),
		
		// Sprites

		sprite: (src,dest,sass) => {
			var spriteDirs = fs.readdirSync(src).filter((file) => fs.statSync(path.join(src, file)).isDirectory());
	
			return merge.apply(null,spriteDirs.map((folder) => {
				// if sprite folder contains PNGs, create a PNG sprite image (to preserve any transparency) if not go for JPG.
				var spriteType = globule.find(src+folder+'/*.png').length ? '.png' : '.jpg',
					spriteData = gulp.src(src+folder+'/*.{png,jpg}',{read:false})
					.pipe($.spritesmith({
						imgName: folder+spriteType,
						imgPath: src+folder+spriteType,
						engine: gmsmith,
						imgOpts: {quality: 97},
						cssName: '_'+folder+'.scss',
						cssSpritesheetName: folder,
						cssVarMap: function (sprite) {
							sprite.name = folder+'-'+sprite.name;
						},

						cssTemplate: './sprites.scss.handlebars',
					}));
				
				
				var imgStream = spriteData.img
					.pipe(buffer())
					.pipe($.imagemin({progressive: true}))		
					.pipe(gulp.dest(dest));	// [main] : create sprite.png/sprite.jpg
				var cssStream = spriteData.css
					.pipe(gulp.dest(sass));	// [main] : create sprite Sass file

				return merge(imgStream, cssStream);
			}));
		},
		
		// Google Fonts
		
		font: (dest,sass) => {
			
			// Download any google fonts
			
			if(globule.find('./google-font.json').length == 0) {
				console.log('Loading all fonts');
				var all = require('google-fonts-complete');
				fs.writeFileSync('./google-font.json',JSON.stringify(all));
			} else {
				console.log('Fonts already exist '+globule.find('./google-font.json').length);
				var all = JSON.parse(fs.readFileSync('./google-font.json'));
			}
			
			var fonts = options.fonts || {},
				fontTypes = {
					"eot": "embedded-opentype",
					"woff": "woff",
					"woff2": "woff2",
					"ttf": "truetype",
					"svg": "svg",
				},
				fontFiles = {},
				fontCSS = [];
			
			//compare fonts you want to the fonts you have
			
			_.each(fonts,(a,name) => {
				var f = all[name].variants || {};
				if(_.isBoolean(a)) {
					var types = _.flatten(_.map(f,(b,c) => _.map(b, (d,e) => c + e)));
				} else if(_.isString(a) || _.isNumber(a)) {
					var types = a.toString().split(',');
				} else {
					var types = a;
				}
				
				_.each(types, (type) => {
					var style = type.match(/italic/) ? 'italic' : 'normal',
						weight = type.replace(/[^0-9]/g,''),
						data = f[style][weight],
						filename = data.local[1].replace(/[^a-zA-Z0-9\-]/g,''),
						urls = data.url,
						fontLocal = data.local.map((a) => 'local('+a+')');
					
					// add in URLs to download
					_.each(urls, (url,ext) => {
						if(globule.find('../dist/fonts/'+filename+'.'+ext).length == 0) fontFiles[filename+'.'+ext] = url;
						fontLocal.push('url("../fonts/'+filename+'.'+ext+'") format("'+fontTypes[ext]+'")');
					});
					
					// add in font-face CSS
					fontCSS.push('@font-face {'
						+ 'font-family: "'+name+'";'
						+ 'font-style: '+style+';'
						+ 'font-weight: '+weight+';'
						+ 'src: '+fontLocal.join()+';'
						+ '}');
					
				});
			});
			
			//write CSS
			
			fs.writeFileSync(sass,fontCSS.join('\n'));
			
			return $.downloadFiles(fontFiles).pipe(gulp.dest(dest));

		},
		
		
		
		// HTML
		
		html: (src,dest,build) => gulp.src(src)
			.pipe($.plumber())
			//.pipe($.changed(dest))
			.pipe($.replaceInclude({
				src: '../src/',
				include: options.include,
				global: _.extend({
					build: build,
					site: options.site
				},options[build] || {}),
			}))
			.pipe(through.obj((file, enc, cb) => {
				var text = file.contents.toString(),
					html = htmlmin(text, {
						collapseWhitespace: true,
						conservativeCollapse:true,
						minifyCSS: true,					
					});
				file.contents = new Buffer(html);
				cb(null,file);
			}))
			.pipe(gulp.dest(dest)),

		// PHP

		php: (src,dest,build) => gulp.src(src)
			.pipe($.plumber())
			//.pipe($.changed(dest))
			.pipe($.replaceInclude({
				src: '../src/',
				include: options.include,
				global: _.extend({
					build: build,
					site: options.site
				},options[build] || {}),
			}))
			.pipe(through.obj((file, enc, cb) => {
				var text = file.contents.toString(),
					php = text.split(/(<\?php[\s\S]*?\?>)/),
					phpTags = php.filter(function(a) {
						return a.match(/^<\?php[\s\S]*\?>$/);
					}),
					htmlTags = php.map(function(a) {
						if( a.match(/^<\?php[\s\S]*\?>$/) ) return '_phptag_';
						else return a;
					}),
					htmlText = htmlmin(htmlTags.join(''),{
						collapseWhitespace: true,
						conservativeCollapse:true,
						minifyCSS: true,
					}),
					html = htmlText.split('_phptag_').map(function(a,b) {
						if(phpTags.length > b) return a + phpTags[b];
						else return a;
					}).join('');

				file.contents = new Buffer(html);
				cb(null, file);
			}))
			.pipe(gulp.dest(dest)),

		// Javascript	
		
		js: (src,dest,build) => gulp.src(src)
			.pipe($.plumber())							// error handler (plumber prevents gulp from exiting if there is an error)
			//.pipe($.changed(dest))						// only update new files (changed looks at file modified date)
			.pipe($.replaceInclude({
				src: options.src,
				include: options.include,
				global: _.extend({
					build: build,
					site: options.site
				},options[build] || {}),
			}))
			.pipe($.sourcemaps.init())					// create sourcemap
			.pipe($.babel({ presets: [es2015] }))		// [main] : transpiler
			.pipe($.uglify())							// [main] : minify javascript
			.pipe($.sourcemaps.write('./maps'))			// write sourcemap to 'js/maps' folder
			.pipe(gulp.dest(dest)),

		// Sass
			
		sass: (src,dest,build) => gulp.src(src)
			.pipe($.plumber())							// error handling
			.pipe($.replaceInclude({
				src: src,
				include: options.include,
				global: _.extend({
					build: build,
					site: options.site
				},options[build] || {}),
			}))
			.pipe($.sourcemaps.init())					// create sourcemap		
			.pipe($.sass())								// [main] : compile sass
			.pipe($.autoprefixer('last 10 versions'))	// [main] : add vendor prefixes
			.pipe($.shorthand())
			.pipe($.csso())								// [main] : minify CSS
			.pipe($.sourcemaps.write('./maps'))			// write sourcemap to 'css/maps' folder
			.pipe(gulp.dest(dest)),
	};
	
};
